@@ -1,19 +1,20 @@
 import pathlib
+import subprocess
+import warnings
+from collections import defaultdict
+from concurrent.futures import ProcessPoolExecutor, as_completed
+
+import dask
+import joblib
 import numpy as np
 import pandas as pd
-import xarray as xr
-import pybedtools
-import dask
-import subprocess
-from collections import defaultdict
-from ALLCools.mcds import RegionDS
 import pyBigWig
-import warnings
-from concurrent.futures import ProcessPoolExecutor, as_completed
+import pybedtools
+import xarray as xr
 from sklearn.model_selection import train_test_split
-from ALLCools.mcds.utilities import write_ordered_chunks, update_dataset_config
-import joblib
 
+from ALLCools.mcds import RegionDS
+from ALLCools.mcds.utilities import write_ordered_chunks, update_dataset_config
 
 
 def _create_train_region_ds(reptile):
@@ -209,6 +210,49 @@ def _predict_sample(
         return output_path
 
 
+def _call_enhancer_region(bw_path, dmr_bed, threshold, merge_dist, chrom_size_path):
+    high_score_regions = []
+    with pyBigWig.open(bw_path) as bw:
+        for chrom, length in bw.chroms().items():
+            intervals = bw.intervals(chrom)
+            cur_start = None
+            cur_end = None
+            if intervals is None:
+                continue
+            for start, end, score in intervals:
+                if score > threshold:
+                    # high score met, start or extend current enhancer
+                    if cur_start is None:
+                        # init
+                        cur_start = start
+                        cur_end = end
+                    else:
+                        # update end
+                        cur_end = end
+                else:
+                    # small score met, check and save previous enhancer
+                    if cur_start is None:
+                        continue
+                    else:
+                        # save region
+                        high_score_regions.append([chrom, cur_start, cur_end])
+                        cur_start = None
+                        cur_end = None
+            # last region
+            if cur_start is not None:
+                high_score_regions.append([chrom, cur_start, cur_end])
+
+        high_score_regions = pd.DataFrame(high_score_regions, columns=['chrom', 'start', 'end'])
+        # merge close enhancers, annotate DMR ids
+        high_score_regions = pybedtools.BedTool \
+            .from_dataframe(high_score_regions) \
+            .sort(g=chrom_size_path) \
+            .merge(d=merge_dist) \
+            .map(b=dmr_bed, c=4, o='collapse', g=chrom_size_path) \
+            .to_dataframe()
+    return high_score_regions
+
+
 class REPTILE:
     def __init__(
             self,
@@ -238,6 +282,9 @@ class REPTILE:
         # for final prediction results
         self.bigwig_dir = f"{self.output_path}/bigwig"
         pathlib.Path(self.bigwig_dir).mkdir(exist_ok=True)
+        # for final enhancer bed
+        self.enhancer_dir = f"{self.output_path}/enhancer"
+        pathlib.Path(self.enhancer_dir).mkdir(exist_ok=True)
 
         self.train_regions = train_regions
         self.dmr_regions = dmr_regions
@@ -624,10 +671,16 @@ class REPTILE:
             self._dmr_prediction = final_da
         return
 
-    def predict(self, cpu, mask_cutoff=0.3, bw_bin_size=50):
+    def predict(self, cpu, mask_cutoff=0.3, bw_bin_size=10, enhancer_cutoff=0.7):
         self._predict(region_dim="query-region", cpu=cpu, mask_cutoff=mask_cutoff)
         self._predict(region_dim="query-dmr", cpu=cpu, mask_cutoff=mask_cutoff)
-        self.dump_bigwigs(cpu=cpu, mask_cutoff=mask_cutoff, bw_bin_size=bw_bin_size)
+
+        # loading all region ids is quite memory intensive
+        # besides, this step is not time-consuming
+        self.dump_bigwigs(cpu=min(cpu, 5), mask_cutoff=mask_cutoff, bw_bin_size=bw_bin_size)
+
+        # call enhancer with enhancer_cutoff, merge enhancer within one step size
+        self.call_enhancers(threshold=enhancer_cutoff, merge_dist=self.step_size)
         return
 
     def _dump_sample(self, sample, mask_cutoff, bw_bin_size):
@@ -666,6 +719,19 @@ class REPTILE:
                 header=None,
             )
 
+            # use bedtools unionbedg to aggregate scores
+            subprocess.run(
+                f"bedtools unionbedg -i "
+                f"{self.bigwig_dir}/{sample}_dmr_pred.bg "
+                f"{self.bigwig_dir}/{sample}_region_pred.bg > "
+                f"{self.bigwig_dir}/{sample}_unionbedg.tsv",
+                shell=True,
+                check=True)
+
+            union_table = pd.read_csv(f'{self.bigwig_dir}/{sample}_unionbedg.tsv',
+                                      sep='\t',
+                                      header=None)
+
             bw_path = f"{self.bigwig_dir}/{sample}_reptile_score.bw"
             with pyBigWig.open(bw_path, "w") as bw:
                 chrom_sizes = pd.read_csv(
@@ -674,75 +740,64 @@ class REPTILE:
                     index_col=0,
                     header=None,
                     squeeze=True,
-                ).to_dict()
+                ).sort_index()
+
                 bw.addHeader(
-                    [(k, v) for k, v in pd.Series(chrom_sizes).sort_index().items()]
+                    [(k, v) for k, v in chrom_sizes.items()]
                 )
 
-                p = subprocess.run(
-                    f"bedtools unionbedg -i "
-                    f"{self.bigwig_dir}/{sample}_dmr_pred.bg "
-                    f"{self.bigwig_dir}/{sample}_region_pred.bg",
-                    shell=True,
-                    check=True,
-                    stdout=subprocess.PIPE,
-                    encoding="utf8",
-                )
+                for chrom in chrom_sizes.index:
+                    cur_bin = 0
+                    cur_scores = [0]
+                    for _, row in union_table[union_table[0] == chrom].iterrows():
+                        chrom, start, end, *scores = row
+                        score = max(map(float, scores))
+                        start_bin = int(start) // bw_bin_size
+                        end_bin = int(end) // bw_bin_size + 1
 
-                cur_bin = 0
-                cur_scores = [0]
-                for line in p.stdout.split("\n"):
-                    if line == "":
-                        continue
-                    if line[-1] == ".":
-                        # no score
-                        continue
+                        for bin_id in range(start_bin, end_bin):
+                            if bin_id > cur_bin:
+                                # save previous bin
+                                cur_pos = cur_bin * bw_bin_size
+                                mean_score = sum(cur_scores) / len(cur_scores)
+                                try:
+                                    if mean_score > 0.01:
+                                        bw.addEntries(
+                                            chrom,
+                                            [cur_pos],
+                                            values=[mean_score],
+                                            span=bw_bin_size,
+                                        )
+                                except RuntimeError as e:
+                                    print(chrom, cur_pos, mean_score, bw_bin_size)
+                                    raise e
 
-                    chrom, start, end, *scores = line.split("\t")
-                    score = max(map(float, scores))
-                    start_bin = int(start) // bw_bin_size
-                    end_bin = int(end) // bw_bin_size + 1
+                                # init new bin
+                                cur_bin = bin_id
+                                cur_scores = [score]
+                            elif bin_id == cur_bin:
+                                # the same bin, take average
+                                cur_scores.append(score)
+                            else:
+                                # no score, initial state
+                                pass
 
-                    for bin_id in range(start_bin, end_bin):
-                        if bin_id > cur_bin:
-                            # save previous bin
-                            cur_pos = cur_bin * bw_bin_size
-                            mean_score = sum(cur_scores) / len(cur_scores)
-                            try:
-                                bw.addEntries(
-                                    chrom,
-                                    [cur_pos],
-                                    values=[mean_score],
-                                    span=bw_bin_size,
-                                )
-                            except RuntimeError as e:
-                                print(chrom, cur_pos, mean_score, bw_bin_size)
-                                raise e
-
-                            # init new bin
-                            cur_bin = bin_id
-                            cur_scores = [score]
-                        elif bin_id == cur_bin:
-                            # the same bin, take average
-                            cur_scores.append(score)
-                        else:
-                            # no score, initial state
-                            pass
-
-                # final
-                cur_pos = cur_bin * bw_bin_size
-                mean_score = sum(cur_scores) / len(cur_scores)
-                try:
-                    bw.addEntries(
-                        chrom, [cur_pos], values=[mean_score], span=bw_bin_size
-                    )
-                except RuntimeError as e:
-                    print(chrom, cur_pos, mean_score, bw_bin_size)
-                    raise e
+                    # final
+                    cur_pos = cur_bin * bw_bin_size
+                    mean_score = sum(cur_scores) / len(cur_scores)
+                    try:
+                        if mean_score > 0.01:
+                            bw.addEntries(
+                                chrom, [cur_pos], values=[mean_score], span=bw_bin_size
+                            )
+                    except RuntimeError as e:
+                        print(chrom, cur_pos, mean_score, bw_bin_size)
+                        raise e
 
             subprocess.run(
                 f"rm -f {self.bigwig_dir}/{sample}_dmr_pred.bg "
-                f"{self.bigwig_dir}/{sample}_region_pred.bg",
+                f"{self.bigwig_dir}/{sample}_region_pred.bg "
+                f"{self.bigwig_dir}/{sample}_unionbedg.tsv",
                 shell=True,
             )
         return bw_path
@@ -769,3 +824,22 @@ class REPTILE:
                 bw_path = f.result()
                 print(bw_path)
         return
+
+    def call_enhancers(self, threshold=0.7, merge_dist=None):
+        if merge_dist is None:
+            merge_dist = self.step_size
+
+        print(f'Call final enhancer with threshold {threshold}, merge enhancer within {merge_dist}bp')
+        dmr_bed = pybedtools.BedTool(self.dmr_regions).sort(g=self.chrom_size_path)
+
+        for sample in self.samples:
+            bw_path = f"{self.bigwig_dir}/{sample}_reptile_score.bw"
+            enhancer_df = _call_enhancer_region(bw_path=bw_path,
+                                                dmr_bed=dmr_bed,
+                                                threshold=threshold,
+                                                merge_dist=merge_dist,
+                                                chrom_size_path=self.chrom_size_path)
+            sample_out_path = f"{self.enhancer_dir}/{sample}_enhancer.bed"
+            enhancer_df.to_csv(sample_out_path, sep='\t', index=None, header=None)
+            print(f'Sample {sample} has {enhancer_df.shape[0]} enhancers.')
+        return
